@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2010 Dave M
+# ClamTk, copyright (C) 2004-2011 Dave M
 #
 # This file is part of ClamTk.
 #
@@ -13,15 +13,15 @@ package ClamTk::App;
 
 use strict;
 #use warnings;    # disabled upon release
-$|++;
-
-use Net::DNS;
+$| = 1;
 
 use encoding 'utf8';
 
+use Net::DNS;
+
 sub get_TK_version {
     # Stick with %.2f format - 4.50 vice 4.5
-    return '4.30';
+    return '4.32';
 }
 
 sub get_path {
@@ -96,10 +96,14 @@ sub def_paths {
 
     my ( $DAILY_PATH, $MAIN_PATH );
 
-    # These are the typical directories where the sigs are found:
+    # These are the typical directories where the sigs are found.
+	# Because CentOS is a little screwy, it will often contain two
+	# directories of definitions... The newer one is likely in
+	# /var/clamav, so check that first.  Other distros will skip it
+	# and will likely find the defs under /var/lib/clamav.
     my @dirs = qw#
-        /var/lib/clamav
         /var/clamav
+        /var/lib/clamav
         /opt/local/share/clamav
         /usr/share/clamav
         /usr/local/share/clamav
@@ -136,9 +140,11 @@ sub def_paths {
 }
 
 sub get_AV_version {
-    # simple 'clamscan -V'
-    # We have to parse this:
+    # simple 'clamscan -V'.
+    # We have to parse something like this:
     # ClamAV 0.95.3/11220/Fri Jun 18 22:06:39 2010
+    # Worth keeping an eye on since it's changed in
+    # past without me noticing...
     local $ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin';
     delete @ENV{ 'IFS', 'CDPATH', 'ENV', 'BASH_ENV' };
     my $paths = ClamTk::App->get_path('clampath');
