@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2010 Dave M
+# ClamTk, copyright (C) 2004-2011 Dave M
 #
 # This file is part of ClamTk.
 #
@@ -21,17 +21,13 @@ package ClamTk::Device;
 
 use strict;
 #use warnings;    # disabled upon release
-$|++;
-
-use Cwd 'realpath';
+$| = 1;
 
 use Glib 'TRUE', 'FALSE';
-use Gtk2 '-init';
-
-#use ClamTk::GUI;
 
 use encoding 'utf8';
 
+use Cwd 'realpath';
 use Locale::gettext;
 use POSIX 'locale_h';
 textdomain('clamtk');
@@ -96,11 +92,11 @@ sub look_for_device {
     # an extra label skews the display.
     $tt->set_tip( $dframe, gettext('Select a device or press Cancel') );
 
-    # This buttonbox will hold the devices found.
-    my $m_bbox = Gtk2::HButtonBox->new;
-    $dframe->add($m_bbox);
-    $m_bbox->set_layout('start');
-    $m_bbox->set_border_width(10);
+    # This toolbar will hold the devices found.
+    my $m_tb = Gtk2::Toolbar->new;
+    $m_tb->set_style('both');
+    $m_tb->set_show_arrow(FALSE);
+    $dframe->add($m_tb);
 
     # Get the information from the devices for display
     for my $d ( keys %devices ) {
@@ -108,27 +104,24 @@ sub look_for_device {
         my ($mount)  = $devices{$d}{mount};
         my ($device) = $devices{$d}{label};
 
-        # We'll use the same technique for images -
-        # stock gtk2 icons, but the label is customized
+        # Now figure out which icon we need, based on the device found.
         my $gui_img = Gtk2::Image->new_from_stock(
               ( $device eq $cd_label )     ? 'gtk-cdrom'
             : ( $device eq $usb_label )    ? 'gtk-harddisk'
             : ( $device eq $floppy_label ) ? 'gtk-floppy'
             : '',
-            'small-toolbar'
+            'large-toolbar'
         );
-        my $gui_btn = Gtk2::Button->new();
-        $gui_btn->set_property( 'image' => $gui_img );
-        $gui_btn->set_label($label);
-        $gui_btn->set_relief('half');
+        my $gui_btn = Gtk2::ToolButton->new( $gui_img, $label );
+        $gui_btn->set_expand(TRUE);
         $tt->set_tip( $gui_btn, "$device ($mount)" );
-        $m_bbox->add($gui_btn);
         $gui_btn->signal_connect(
             clicked => sub {
                 $dwin->destroy;
                 ClamTk::GUI->getfile( 'device', $mount );
             }
         );
+        $m_tb->insert( $gui_btn, -1 );
     }
 
     # This hbuttonbox holds the cancel button.
