@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2010 Dave M
+# ClamTk, copyright (C) 2004-2011 Dave M
 #
 # This file is part of ClamTk.
 #
@@ -13,18 +13,10 @@ package ClamTk::GUI;
 
 use strict;
 #use warnings;    # disabled upon release
-$|++;
+$| = 1;
 
 use encoding 'utf8';
 
-use ClamTk::App;
-use ClamTk::Prefs;
-use ClamTk::Results;
-use ClamTk::Update;
-use ClamTk::Schedule;
-use ClamTk::Device;
-use ClamTk::Submit;
-
 use Cwd 'abs_path';
 use File::Basename 'basename', 'dirname';
 use POSIX 'locale_h',          'strftime';
@@ -45,7 +37,6 @@ use Glib 'TRUE', 'FALSE';
 my $window;
 my $top_label;
 
-my ( $save_log, $recursive, $hidden, $size_set, $thorough ) = (0) x 5;
 my ( @files, @quoted, @new );
 my %dirs_scanned;
 my $step = 0;    # step for progressbar
@@ -68,8 +59,6 @@ my ( $engine_gui_img, $defs_gui_img, $status_gui_img );
 my ($scan_status_label);
 my $av_remote = '';
 
-my $actions;
-
 my ( @q_files,   @h_files );
 my ( $q_label,   $h_label );
 my ( $new_slist, $new_hlist );
@@ -109,48 +98,43 @@ sub start_gui {
         # Scan a single file
         [   'Scan_File',        'gtk-copy',
             gettext('A _File'), '<control>F',
-            gettext('Scan a file'), sub { ClamTk::GUI->getfile('file') },
+            undef, 	sub { ClamTk::GUI->getfile('file') },
 	    FALSE
         ],
         # Scan home directory, but do not descend
         [   'Quick_Home', 		'gtk-go-down',
             gettext('Home (_Quick)'), 	'<control>Q',
-            gettext('Quick Home Scan'),
-            sub { ClamTk::GUI->getfile('home') },
+            undef, 	sub { ClamTk::GUI->getfile('home') },
 	    FALSE
         ],
         # Scan home directory, descend (recursive)
         [   'Full_Home', 		'gtk-goto-bottom',
             gettext('Home (Recursive)'), 	'<control>Z',
-            gettext('Full Home Scan'),
-            sub { ClamTk::GUI->getfile('full-home') },
+            undef,	 sub { ClamTk::GUI->getfile('full-home') },
 	    FALSE
         ],
         # Scan a single directory
         [   'Scan_Directory', 		'gtk-directory',
             gettext('A _Directory'), 	'<control>D',
-            gettext('Scan a Directory'),
-            sub { ClamTk::GUI->getfile('dir') },
+            undef,	sub { ClamTk::GUI->getfile('dir') },
 	    FALSE
         ],
         # Scan a directory, descend (recursive)
         [   'Recursive_Scan', 		'gtk-sort-ascending',
             gettext('_Recursive Scan'), '<control>R',
-            gettext('Recursively scan a directory'),
-            sub { ClamTk::GUI->getfile('recur') },
+            undef,	sub { ClamTk::GUI->getfile('recur') },
 	    FALSE
         ],
         # Scan a device - relies upon udev and /proc/mounts
         [   'Scan_Device', 		'gtk-cdrom',
             gettext('A Device'), 	'<control>J',
-            gettext('Scan a Device'),
-            sub { ClamTk::Device->look_for_device() },
+            undef,	sub { ClamTk::Device->look_for_device() },
 	    FALSE
         ],
         # Quit
         [   'Exit',           'gtk-quit',
             gettext('E_xit'), '<control>X',
-            gettext('Quit this program'), sub { Gtk2->main_quit },
+            undef,	sub { Gtk2->main_quit },
 	    FALSE
         ],
 
@@ -158,37 +142,33 @@ sub start_gui {
         # Show dialog of recorded scans
         [   'ManageHistories', 		'gtk-index',
             gettext('Manage _Histories'), "<control>H",
-            gettext('Select Histories to Delete'),
-            sub { history('delete') },
+            undef, 	sub { history('delete') },
             FALSE
         ],
         # Closes the scan bar, zeroes out everything
         [   'ClearOutput',                'gtk-clear',
             gettext('Clear _Output'),     '<control>O',
-            gettext('Clear the Display'), \&clear_output,
+            undef,	\&clear_output,
             FALSE
         ],
 
         ### These fall under Quarantine ###
         # Simple window showing number of quarantined objects
         [   'Status', 			'gtk-edit', 
-	     gettext('_Status'), 	'<control>S',
-            gettext('See how many files are quarantined'),
-            \&quarantine_check,
+	    gettext('_Status'), 	'<control>S',
+            undef,	\&quarantine_check,
 	    FALSE
         ],
         # Dialog box - delete or return quarantined objects
         [   'Maintenance', 		'gtk-preferences',
             gettext('_Maintenance'), 	'<control>M',
-            gettext('View files that have been quarantined'),
-            \&maintenance,
+            undef,	\&maintenance,
 	    FALSE
         ],
         # Delete quarantined objects
         [   'Empty', 			'gtk-delete',
             gettext('_Empty Quarantine Folder'), '<control>E',
-            gettext('Delete all files that have been quarantined'),
-            \&del_quarantined,
+            undef,	\&del_quarantined,
 	    FALSE
         ],
 
@@ -196,14 +176,13 @@ sub start_gui {
         # Dialog box to update signatures, check for GUI updates
         [   'UpdateSig', 			'gtk-network',
             gettext('_Check for updates'), 	'<control>U',
-            gettext('Check for updates'),
-            sub { ClamTk::Update->update_dialog() },
+	    undef,	sub { ClamTk::Update->update_dialog() },
 	    FALSE
         ],
         # Standard About dialog
         [   'About',                          'gtk-about',
             gettext("_About"),                '<control>A',
-            gettext('About this program...'), \&about,
+            undef,	\&about,
 	    FALSE
         ],
 
@@ -211,29 +190,26 @@ sub start_gui {
         # Schedule scans and AV sig updates
         [   'Scheduler', 		'gtk-media-forward',
             gettext('_Scheduler'), 	'<control>T',
-            gettext('Schedule a daily scan'),
-            sub { ClamTk::Schedule->schedule_dialog() },
+	    undef,	sub { ClamTk::Schedule->schedule_dialog() },
 	    FALSE
         ],
         # Change to/from user updates to system
         [   "AVsetup", 			'gtk-refresh',
             gettext('Rerun antivirus setup _wizard'),
             '<control>W',
-            gettext('Rerun antivirus setup wizard'),
-            sub { ClamTk::Update->av_db_select() },
+            undef,	sub { ClamTk::Update->av_db_select() },
 	    FALSE
         ],
         # Huge preferences window
         [   'Preferences',          'gtk-properties',
             gettext('Preferences'), '<control>P',
-            gettext('Preferences'), \&preferences,
+            undef,	\&preferences,
 	    FALSE
         ],
 	# Submit a file for analysis
         [   'SubmitFile',          'gtk-convert',
             gettext('Submit a file for analysis'), '<control>Y',
-            gettext('Submit a file for analysis'),
-	    sub { ClamTk::Submit->analysis() },
+	    undef,	sub { ClamTk::Submit->analysis() },
 	    FALSE
         ],
     );
@@ -274,7 +250,7 @@ sub start_gui {
         </menubar>
 </ui>";
 
-    $actions = Gtk2::ActionGroup->new('Actions');
+    my $actions = Gtk2::ActionGroup->new('Actions');
     $actions->add_actions( \@entries, undef );
 
     my $ui = Gtk2::UIManager->new;
@@ -290,69 +266,68 @@ sub start_gui {
     $top_label = Gtk2::Label->new;
     $vbox->pack_start( $top_label, FALSE, FALSE, 5 );
 
-##########################################################
-## Actions Frame 					##
-## These are for easy access to buttons for scanning 	##
-##########################################################
+    # Actions Frame
+    # These are for easy access to buttons for scanning
 
     my $gui_frame = Gtk2::Frame->new( gettext('Actions') );
     $vbox->pack_start( $gui_frame, FALSE, FALSE, 0 );
 
-    my $gui_box = Gtk2::VBox->new();
+    my $gui_box = Gtk2::VBox->new( TRUE, 5 );
     $gui_frame->add($gui_box);
 
-    my $bbox = Gtk2::HButtonBox->new;
-    $gui_box->pack_start( $bbox, TRUE, TRUE, 0 );
+    # Now that Gnome has decided to not show the images
+    # in buttons by default, the regular buttons looked pretty stupid.
+    # We'll fix the new packing problem by changing the buttonbox
+    # to a toolbar and using those buttons.  Nice try, Gnome.
 
-    # Using gtk2's icons rather than including our own, we
-    # can keep the package smaller. The good news is that we can
-    # borrow the icon and customize its associated text.
+    my $gui_tb = Gtk2::Toolbar->new;
+
+    # In an upcoming version, we'll give the option to
+    # display icons OR just text.
+    $gui_tb->set_style('both');
+
+    $gui_tb->set_border_width(5);
 
     # Button for quick home folder scan
-    my $home_img = Gtk2::Image->new_from_stock( 'gtk-home', 'small-toolbar' );
-    my $home_btn = Gtk2::Button->new();
-    $home_btn->signal_connect( clicked => sub { ClamTk::GUI->getfile('home') }
+    my $home_img = Gtk2::ToolButton->new_from_stock('gtk-home');
+    $home_img->set_expand(TRUE);
+    $home_img->set_label( gettext('Home') );
+    $home_img->signal_connect( clicked => sub { ClamTk::GUI->getfile('home') }
     );
-    $home_btn->set_property( 'image' => $home_img );
-    $home_btn->set_label( gettext('Home') );
-    $tt->set_tip( $home_btn, gettext('Scan your home directory') );
-    $bbox->add($home_btn);
+    $tt->set_tip( $home_img, gettext('Scan your home directory') );
+    $gui_tb->insert( $home_img, -1 );
 
     # Button for scanning just a file
-    my $file_img = Gtk2::Image->new_from_stock( 'gtk-copy', 'small-toolbar' );
-    my $file_btn = Gtk2::Button->new();
-    $file_btn->signal_connect( clicked => sub { ClamTk::GUI->getfile('file') }
+    my $file_img = Gtk2::ToolButton->new_from_stock('gtk-file');
+    $file_img->set_expand(TRUE);
+    $file_img->set_label( gettext('File') );
+    $file_img->signal_connect( clicked => sub { ClamTk::GUI->getfile('file') }
     );
-    $file_btn->set_property( 'image' => $file_img );
-    $file_btn->set_label( gettext('File') );
-    $tt->set_tip( $file_btn, gettext('Scan a file') );
-    $bbox->add($file_btn);
+    $tt->set_tip( $file_img, gettext('Scan a file') );
+    $gui_tb->insert( $file_img, -1 );
 
     # Button for scanning a directory
-    my $dir_img =
-        Gtk2::Image->new_from_stock( 'gtk-directory', 'small-toolbar' );
-    my $dir_btn = Gtk2::Button->new();
-    $dir_btn->signal_connect( clicked => sub { ClamTk::GUI->getfile('dir') }
+    my $dir_img = Gtk2::ToolButton->new_from_stock('gtk-directory');
+    $dir_img->set_expand(TRUE);
+    $dir_img->set_label( gettext('Directory') );
+    $dir_img->signal_connect( clicked => sub { ClamTk::GUI->getfile('dir') }
     );
-    $dir_btn->set_property( 'image' => $dir_img );
-    $dir_btn->set_label( gettext('Directory') );
-    $tt->set_tip( $dir_btn, gettext('Scan a directory') );
-    $bbox->add($dir_btn);
+    $tt->set_tip( $dir_img, gettext('Scan a directory') );
+    $gui_tb->insert( $dir_img, -1 );
 
     # Button to exit
-    my $quit_img = Gtk2::Image->new_from_stock( 'gtk-quit', 'small-toolbar' );
-    my $quit_btn = Gtk2::Button->new();
-    $quit_btn->set_property( 'image' => $quit_img );
-    $quit_btn->set_label( gettext('Exit') );
-    $quit_btn->signal_connect( clicked => sub { Gtk2->main_quit } );
-    $tt->set_tip( $quit_btn, gettext('Exit this program') );
-    $bbox->add($quit_btn);
+    my $quit_img = Gtk2::ToolButton->new_from_stock('gtk-quit');
+    $quit_img->set_expand(TRUE);
+    $quit_img->set_label( gettext('Exit') );
+    $quit_img->signal_connect( clicked => sub { Gtk2->main_quit } );
+    $tt->set_tip( $quit_img, gettext('Exit this program') );
+    $gui_tb->insert( $quit_img, -1 );
 
-##########################################################
-## Status frame 					##
-## This shows versions of ClamAV, ClamTk, signatures, 	##
-## date of last scan, date of last infected file 	##
-##########################################################
+    $gui_box->pack_start( $gui_tb, TRUE, TRUE, 0 );
+
+    # Status frame
+    # This shows versions of ClamAV, ClamTk, signatures,
+    # date of last scan, date of last infected file
 
     my $status_frame = Gtk2::Frame->new( gettext('Status') );
     $vbox->pack_start( $status_frame, FALSE, FALSE, 0 );
@@ -365,7 +340,7 @@ sub start_gui {
     my $engine_text = Gtk2::Label->new( gettext('Antivirus engine') );
     $engine_version_text = Gtk2::Label->new( gettext('Unknown') );
     $engine_gui_img =
-        Gtk2::Image->new_from_stock( 'gtk-apply', 'small-toolbar' );
+        Gtk2::Image->new_from_stock( 'gtk-yes', 'small-toolbar' );
     $tt->set_tip( $engine_gui_img, gettext('Current') );
     $engine_gui_box->pack_start( $engine_gui_img,      TRUE, TRUE, 0 );
     $engine_gui_box->pack_start( $engine_text,         TRUE, TRUE, 0 );
@@ -377,7 +352,7 @@ sub start_gui {
     my $gui_text         = Gtk2::Label->new( gettext('GUI version') );
     my $gui_version_text = Gtk2::Label->new( ClamTk::App->get_TK_version() );
     $status_gui_img =
-        Gtk2::Image->new_from_stock( 'gtk-apply', 'small-toolbar' );
+        Gtk2::Image->new_from_stock( 'gtk-yes', 'small-toolbar' );
     $tt->set_tip( $status_gui_img, gettext('Current') );
     $status_gui_box->pack_start( $status_gui_img,   TRUE, TRUE, 0 );
     $status_gui_box->pack_start( $gui_text,         TRUE, TRUE, 0 );
@@ -388,8 +363,7 @@ sub start_gui {
     my $defs_gui_box = Gtk2::HBox->new( TRUE, 0 );
     my $defs_text = Gtk2::Label->new( gettext('Antivirus definitions') );
     $defs_version_text = Gtk2::Label->new( gettext('Unknown') );
-    $defs_gui_img =
-        Gtk2::Image->new_from_stock( 'gtk-apply', 'small-toolbar' );
+    $defs_gui_img = Gtk2::Image->new_from_stock( 'gtk-yes', 'small-toolbar' );
     $tt->set_tip( $defs_gui_img, gettext('Current') );
     $defs_gui_box->pack_start( $defs_gui_img,      TRUE, TRUE, 0 );
     $defs_gui_box->pack_start( $defs_text,         TRUE, TRUE, 0 );
@@ -398,10 +372,10 @@ sub start_gui {
 
     # Last virus scan date
     my $last_gui_box = Gtk2::HBox->new( TRUE, 0 );
-    my $last_gui_img =
-        Gtk2::Image->new_from_stock( 'gtk-print-preview', 'small-toolbar' );
     my $last_text = Gtk2::Label->new( gettext('Last scan') );
     $last_date_text = Gtk2::Label->new( gettext('Unknown') );
+    my $last_gui_img =
+        Gtk2::Image->new_from_stock( 'gtk-find', 'small-toolbar' );
     $tt->set_tip( $last_gui_img, gettext('Date of your last scan') );
     $last_gui_box->pack_start( $last_gui_img,   TRUE, TRUE, 0 );
     $last_gui_box->pack_start( $last_text,      TRUE, TRUE, 0 );
@@ -411,7 +385,7 @@ sub start_gui {
     # Last infected file date
     my $last_infect_box = Gtk2::HBox->new( TRUE, 0 );
     my $last_infect_img =
-        Gtk2::Image->new_from_stock( 'gtk-dnd', 'small-toolbar' );
+        Gtk2::Image->new_from_stock( 'gtk-find', 'small-toolbar' );
     my $last_infect_text = Gtk2::Label->new( gettext('Last infected file') );
     $last_infect_date = Gtk2::Label->new( gettext('Unknown') );
     $tt->set_tip( $last_infect_img, gettext('Date of last known infection') );
@@ -420,11 +394,9 @@ sub start_gui {
     $last_infect_box->pack_start( $last_infect_date, TRUE, TRUE, 0 );
     $status_table->attach_defaults( $last_infect_box, 0, 3, 4, 5 );
 
-##################################################################
-## Scan frame 							##
-## This holds the progressbar and keeps track			##
-## of files scanned, viruses found and the status/elapsed time 	##
-##################################################################
+    # Scan frame
+    # This holds the progressbar and keeps track of files
+    # scanned, viruses found and the status/elapsed time
 
     $scan_frame = Gtk2::Frame->new( gettext('Scan') );
     $vbox->pack_start( $scan_frame, FALSE, FALSE, 0 );
@@ -486,12 +458,10 @@ sub start_gui {
     $right_status = Gtk2::Label->new( gettext('Ready') );
     $bottom_box->pack_start( $right_status, TRUE, TRUE, 5 );
 
+    $window->set_focus_child($stop_btn);
     $window->show_all();
     $scan_frame->hide;
 
-    # Query user's preferences for loading
-    load_prefs();
-
     # The only reason we take @ARGV is for right-click scanning,
     # not arguments (e.g., -v).
     if (@ARGV) {
@@ -586,6 +556,7 @@ sub start_gui {
 }
 
 sub update_status_frame {
+
     # This is useful as its own subroutine so we
     # can call it from anywhere
     $top_label->set_text( gettext('Please wait, initializing...') );
@@ -602,11 +573,10 @@ sub update_status_frame {
             'small-toolbar' );
         $tt->set_tip( $status_gui_img, gettext('Unable to check') );
     } elsif ( $ret == 1 || $ret == 3 ) {
-        $status_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
+        $status_gui_img->set_from_stock( 'gtk-yes', 'small-toolbar' );
         $tt->set_tip( $status_gui_img, gettext('Current') );
     } else {
-        $status_gui_img->set_from_stock( 'gtk-dialog-error',
-            'small-toolbar' );
+        $status_gui_img->set_from_stock( 'gtk-no', 'small-toolbar' );
         $tt->set_tip( $status_gui_img,
             gettext('A newer version is available') );
     }
@@ -649,8 +619,7 @@ sub set_AV_engine_status {
         Gtk2->main_iteration while ( Gtk2->events_pending );
         return ( gettext('Unable to check') );
     } elsif ( ( $version cmp $av_remote ) == -1 ) {
-        $engine_gui_img->set_from_stock( 'gtk-dialog-error',
-            'small-toolbar' );
+        $engine_gui_img->set_from_stock( 'gtk-no', 'small-toolbar' );
         $tt->set_tip( $engine_gui_img,
             gettext('The antivirus engine is reporting an outdated engine') );
         $window->queue_draw;
@@ -658,13 +627,13 @@ sub set_AV_engine_status {
         return (
             gettext('The antivirus engine is reporting an outdated engine') );
     } elsif ( ( $version cmp $av_remote ) == 0 ) {
-        $engine_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
+        $engine_gui_img->set_from_stock( 'gtk-yes', 'small-toolbar' );
         $tt->set_tip( $engine_gui_img, gettext('Current') );
         $window->queue_draw;
         Gtk2->main_iteration while ( Gtk2->events_pending );
         return ( gettext('Current') );
     } elsif ( ( $version cmp $av_remote ) == 1 ) {    # shouldn't happen
-        $engine_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
+        $engine_gui_img->set_from_stock( 'gtk-yes', 'small-toolbar' );
         $tt->set_tip( $engine_gui_img, gettext('Current') );
         $window->queue_draw;
         Gtk2->main_iteration while ( Gtk2->events_pending );
@@ -706,11 +675,11 @@ sub set_sig_status {
         $tt = Gtk2::Tooltips->new();
     }
     if ( $date_ret eq 'outdated' || !$sig_date ) {
-        $defs_gui_img->set_from_stock( 'gtk-dialog-error', 'small-toolbar' );
+        $defs_gui_img->set_from_stock( 'gtk-no', 'small-toolbar' );
         $tt->set_tip( $defs_gui_img,
             gettext('Your antivirus signatures are out-of-date') );
     } else {
-        $defs_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
+        $defs_gui_img->set_from_stock( 'gtk-yes', 'small-toolbar' );
         $tt->set_tip( $defs_gui_img, gettext('Current') );
     }
     return;
@@ -725,11 +694,10 @@ sub set_tk_status {
             'small-toolbar' );
         $tt->set_tip( $status_gui_img, gettext('Unable to check') );
     } elsif ( $ret == 1 || $ret == 3 ) {
-        $status_gui_img->set_from_stock( 'gtk-apply', 'small-toolbar' );
+        $status_gui_img->set_from_stock( 'gtk-yes', 'small-toolbar' );
         $tt->set_tip( $status_gui_img, gettext('Current') );
     } else {
-        $status_gui_img->set_from_stock( 'gtk-dialog-error',
-            'small-toolbar' );
+        $status_gui_img->set_from_stock( 'gtk-no', 'small-toolbar' );
         $tt->set_tip( $status_gui_img,
             gettext('A newer version is available') );
     }
@@ -776,6 +744,7 @@ sub getfile {
     my @cmd_input = @_;
 
     my $paths = ClamTk::App->get_path('all');
+    my %prefs = ClamTk::Prefs->get_all_prefs();
 
     # Don't bother doing anything if clamscan can't be found
     warn "Cannot scan without clamscan!\n" unless $paths->{clampath};
@@ -784,9 +753,9 @@ sub getfile {
     # these lines are for 'thorough' :)
     # If it's selected, we add detection for both
     # potentially unwanted applications and broken executables.
-    if ($thorough) {
+    if ( $prefs{Thorough} ) {
         $paths->{clamscan} .= ' --detect-pua --detect-broken';
-    } elsif ( !$thorough ) {
+    } else {
         $paths->{clamscan} =~ s/\s--detect-pua --detect-broken//;
     }
 
@@ -802,19 +771,61 @@ sub getfile {
     my ( $dir, $dialog );
     Gtk2->main_iteration while ( Gtk2->events_pending );
     my $rule = File::Find::Rule->new;
+
+    # If we filter the results now, F:F:R should be much faster.
+    # We need to add the "not"'s first.
+
+    # By default, we ignore .gvfs directories.
+    # Once we figure out KDE's process, we'll include that too.
+    if ( !ClamTk::Prefs->get_preference('Mounted') ) {
+
+        # This returned a '0', so filter the results
+        for my $m (qw(.gvfs smb4k)) {
+            $rule->not(
+                File::Find::Rule->name($m)->directory->prune->discard );
+        }
+    }
+
+    for my $ignore (
+        split(
+            /;/,
+            ClamTk::Prefs->get_preference('Whitelist')
+                . $paths->{whitelist_dir}
+        )
+        )
+    {
+
+        # We just want the base directory name
+        $ignore = ( split( '/', $ignore ) )[-1];
+        $rule->not(
+            File::Find::Rule->name($ignore)->directory->prune->discard );
+    }
+
+    # Remove mail directories for now -
+    # until we can parse them... sigh.
+    # http://clamtk.sourceforge.net/faq.html#inbox
+    my @maildirs = qw|
+        .thunderbird	.mozilla-thunderbird
+        .evolution	Mail	kmail|;
+    for my $mailbox (@maildirs) {
+        $rule->not(
+            File::Find::Rule->name($mailbox)->directory->prune->discard );
+    }
+
+    # Now add what we're looking for.
     $rule->file;
     $rule->exists;
     $rule->readable;
     $rule->maxdepth(1)
         unless ( $option eq 'recur'
         or $option eq 'full-home'
-        or $recursive );
+        or $prefs{Recursive} );
 
     my $title =
           ( $option eq 'file' ) ? gettext('Select File')
-        : ( $option eq 'dir' && !$recursive )
+        : ( $option eq 'dir' && !$prefs{Recursive} )
         ? gettext('Select a Directory (directory scan)')
-        : ( $option eq 'recur' or $recursive )
+        : ( $option eq 'recur' or $prefs{Recursive} )
         ? gettext('Select a Directory (recursive scan)')
         : '';
 
@@ -925,8 +936,10 @@ sub getfile {
     if ( $option eq 'file' ) {
         scan(@files);
     } else {
+
         # Pass the files to the filter first
         @files = filter( $option, @files );
+
         # the pulse of the progressbar
         if ( scalar(@files) > 1 ) {
             $step = 1 / scalar(@files);
@@ -936,7 +949,7 @@ sub getfile {
 
         # By default, 20Mb is the largest we go -
         # unless the preference is to ignore size.
-        if ( !$size_set ) {
+        if ( !$prefs{SizeLimit} ) {
             my @large;
             foreach my $foo (@files) {
                 if ( -s $foo >= 20_000_000 ) {
@@ -988,8 +1001,13 @@ sub filter {
     my $opt    = shift;
     my @filter = @_;
 
+    my $paths = ClamTk::App->get_path('all');
+
     # remove the hidden files if chosen
-    if ( $hidden == 0 && $opt ne 'recur' && $opt ne 'full-home' ) {
+    if (   !ClamTk::Prefs->get_preference('ScanHidden')
+        && $opt ne 'recur'
+        && $opt ne 'full-home' )
+    {
         @filter = grep { basename($_) !~ /^\./ } @filter;
     }
 
@@ -1003,36 +1021,12 @@ sub filter {
         }
     }
 
-    # Remove whitelisted directories
-    # (unless it's a single file - or option eq 'file')
-    my $paths = ClamTk::App->get_path('all');
-
-    # Remove mail directories for now -
-    # until we can parse them... sigh.
-    # http://clamtk.sourceforge.net/faq.html#inbox
-    my @maildirs = qw|
-        .thunderbird	.mozilla-thunderbird
-        .evolution	Mail	kmail|;
-    for my $mailbox (@maildirs) {
-        $mailbox = "$paths->{directory}/$mailbox";
-        @filter = grep { !m#$mailbox# } @filter;
-    }
-
-    for my $ignore (
-        split(
-            /;/,
-            ClamTk::Prefs->get_preference('Whitelist')
-                . $paths->{whitelist_dir}
-        )
-        )
-    {
-        @filter = grep { !m#$ignore# } @filter;
-    }
     return @filter;
 }
 
 sub scan {
     my @get = @_;
+
     # We need to quotemeta the filenames
     @quoted = map { quotemeta($_) } @get;
     timer();
@@ -1056,6 +1050,7 @@ sub scan {
         ? dirname( $get[$scan_count] )
         : $get[$scan_count]
     );
+
     # this is for the 'stop button'
     $scan_pid = $pid;
 
@@ -1063,9 +1058,8 @@ sub scan {
     while (<$SCAN>) {
         Gtk2->main_iteration while Gtk2->events_pending;
         if ( $av_remote eq 'undef' && /ClamAV engine is outdated/ ) {
-            $engine_gui_img->set_from_stock( 'gtk-dialog-error',
-                'small-toolbar' );
-            #$engine_version_text->set_text( gettext('Outdated') );
+            $engine_gui_img->set_from_stock( 'gtk-no', 'small-toolbar' );
+            $engine_version_text->set_text( gettext('Outdated') );
             $tt->set_tip( $engine_gui_img,
                 gettext('The antivirus is reporting an outdated engine') );
             $window->queue_draw;
@@ -1106,6 +1100,7 @@ sub scan {
         timer();
         my $clean_words = join(
             '|',
+
             # These aren't necessarily clean (despite the scalar name);
             # we just don't want them counted as viruses
             'OK',                          'Zip module failure',
@@ -1173,19 +1168,25 @@ sub scan {
 }
 
 sub clear_output {
+
     # Return if there are files being scanned
     return if ( scalar(@files) > 0 );
+
     # Reset the progressbar
     $pb->set_fraction(0);
+
     # Refresh the main window
     $window->queue_draw;
+
     # Clear the text
     $top_label->set_text('');
     $scan_status_label->set_text('');
+
     # Reset the bottom labels
     $left_status->set_text( gettext('Files Scanned: ') );
     $mid_status->set_text( gettext('Threats Found: ') );
     $right_status->set_text( gettext('Ready') );
+
     # Hide the scanning frame
     $scan_frame->hide();
     return;
@@ -1206,8 +1207,10 @@ sub timer {
 }
 
 sub clean_up {
+
     # Ensure the progressbar has hit 100%...
     $pb->set_fraction(1.0);
+
     # ...and remove the text
     $pb->set_text("");
 
@@ -1227,7 +1230,9 @@ sub clean_up {
     $window->queue_draw;
     Gtk2->main_iteration while ( Gtk2->events_pending );
 
-    if ($save_log) {
+    my %prefs = ClamTk::Prefs->get_all_prefs();
+
+    if ( $prefs{SaveToLog} ) {
         my $paths     = ClamTk::App->get_path('history');
         my $virus_log = $paths . "/" . "$mon-$day-$year" . ".log";
 
@@ -1251,7 +1256,6 @@ sub clean_up {
         } else {
             $scan_status_label->set_text(
                 gettext('Could not write to logfile. Check permissions.') );
-            $save_log = 0;
         }
     }
     $db_total =~ s/(\w+)\s+$/$1/;
@@ -1267,16 +1271,16 @@ sub clean_up {
     $window->queue_draw;
 
     if ( $num_so_far == 0 ) {
-        print $REPORT gettext("No threats found.\n") if ($save_log);
+        print $REPORT gettext("No threats found.\n") if ( $prefs{SaveToLog} );
     } else {
-        if ($save_log) {
+        if ( $prefs{SaveToLog} ) {
             for my $num ( sort keys %$found ) {
                 printf $REPORT "%-38s %38s\n",
                     $found->{$num}->{name}, $found->{$num}->{status};
             }
         }
     }
-    if ($save_log) {
+    if ( $prefs{SaveToLog} ) {
         print $REPORT '-' x 77, "\n";
         close($REPORT);
     }
@@ -1376,6 +1380,10 @@ sub restore {
     my $full_path = $top_dir .= $data->[0];
     return if ( not exists $data->[0] );
 
+    #warn "basename = >", $data->[0], "<\n";
+    #warn "top dir = >$top_dir<\n";
+    #warn "full_path = >$full_path<\n";
+
     my $ctx = Digest::MD5->new;
     open( my $F, '<', $full_path ) or do {
         show_message_dialog( $window, 'warning', 'ok',
@@ -1386,26 +1394,37 @@ sub restore {
     my $md5 = $ctx->hexdigest;
     close($F);
 
+    # By default, the final destination will be $HOME
+    my $final_destination = $paths->{directory} . '/';
+
     my ( $path, $perm ) = ClamTk::Prefs->restore( $md5, 'exists' );
     if ( !$path ) {
-        show_message_dialog( $window, 'warning', 'ok',
-            "Problems opening restore file: $!\n" );
-        return;
+
+        # We didn't receive a path for this file, so
+        # by default it's going $HOME
+        $path = $final_destination . $data->[0];
     }
 
-    # By default, the final destination will be $HOME
-    my $final_destination = $paths->{directory};
+    #warn "path after check =>$path<\n";
 
     if ($path) {
-        if ( -e $path ) {    # another file already exists there
+        if ( -e $path ) {
+
+            # Another file by that name already exists
             $final_destination = $path .= '.restore';
         } else {
+
+            # We can set the given path to our final destination
             $final_destination = $path;
         }
     } else {
-        $final_destination = $paths->{directory} . "/" . $data->[0];
+
+        # We have nothing to go on, so it's going $HOME
+        $final_destination .= $data->[0];
     }
 
+    #warn "full_path = >$full_path<, dest = >$final_destination<\n";
+
     system( 'mv', $full_path, $final_destination ) == 0 or do {
         show_message_dialog( $window, 'warning', 'ok',
             "Couldn't move file to $final_destination: $!\n" );
@@ -1581,6 +1600,7 @@ sub history {
     $new_win->signal_connect( destroy => sub { $new_win->destroy } );
     $new_win->set_default_size( 260, 200 );
     $new_win->set_title( gettext('Scanning Histories') );
+    $new_win->set_position('center');
 
     my $top_dir = $paths . "/";
     my $sortnum = 0;              # 0 = asc, 1 = desc
@@ -1614,6 +1634,7 @@ sub history {
             for my $t (@sort_this) {
                 $t = basename($t);
             }
+            @sort_this = map { s/\.log//; $_ } @sort_this;
             push @{ $new_hlist->{data} }, @sort_this;
         }
     );
@@ -1645,6 +1666,7 @@ sub history {
     $new_vbox->pack_start( $h_label, FALSE, FALSE, 2 );
 
     for my $opt (@h_files) {
+        $opt =~ s/\.log//;
         push @{ $new_hlist->{data} }, basename($opt);
     }
     $new_win->set_position('mouse');
@@ -1668,7 +1690,7 @@ sub history_del_single {
     my $history = ClamTk::App->get_path('history');
 
     my $top_dir   = $history . "/";
-    my $full_path = $top_dir . $data->[0];
+    my $full_path = $top_dir . $data->[0] . '.log';
     return if ( !-e $full_path );
 
     unlink $full_path or do {
@@ -1702,6 +1724,7 @@ sub history_del_all {
         my @not_del;
         my $size = @h_files;
         foreach (@h_files) {
+            @h_files = map {"$_.log"} @h_files;
             unlink($_) or push( @not_del, $_ );
         }
         if ( scalar(@not_del) >= 1 ) {
@@ -1726,7 +1749,7 @@ sub view_box {
 
     my $deref     = $sel[0];
     my $data      = $new_hlist->{data}[$deref];
-    my $full_path = $paths . "/" . $data->[0];
+    my $full_path = $paths . "/" . $data->[0] . '.log';
     return if ( !-e $full_path );
 
     my $view_win =
@@ -1771,28 +1794,6 @@ sub view_box {
     return;
 }
 
-sub load_prefs {
-    my %prefs = ClamTk::Prefs->get_all_prefs();
-    return unless %prefs;
-
-    if ( $prefs{SaveToLog} ) {
-        $save_log = 1;
-    }
-    if ( $prefs{ScanHidden} ) {
-        $hidden = 1;
-    }
-    if ( $prefs{SizeLimit} ) {
-        $size_set = 1;
-    }
-    if ( $prefs{Thorough} ) {
-        $thorough = 1;
-    }
-    if ( $prefs{Recursive} ) {
-        $recursive = 1;
-    }
-    return;
-}
-
 sub preferences {
     my %prefs = ClamTk::Prefs->get_all_prefs();
 
@@ -1800,11 +1801,11 @@ sub preferences {
     $pref->signal_connect( destroy => sub { $pref->destroy } );
     $pref->set_title( gettext('Preferences') );
     $pref->set_border_width(5);
-    $pref->set_position('center-always');
+    $pref->set_position('center');
 
     my $box = Gtk2::VBox->new( FALSE, 0 );
     $pref->add($box);
-    $box->set_size_request( 530, 300 );
+    $box->set_size_request( -1, 300 );
 
     my $nb = Gtk2::Notebook->new;
     $nb->set_scrollable(TRUE);
@@ -1821,7 +1822,6 @@ sub preferences {
         toggled => sub {
             ClamTk::Prefs->set_preference( 'ScanHidden',
                 $hidden_box->get_active ? 1 : 0 );
-            $hidden ^= 1;
         }
     );
 
@@ -1833,7 +1833,6 @@ sub preferences {
         toggled => sub {
             ClamTk::Prefs->set_preference( 'Recursive',
                 $recur_box->get_active ? 1 : 0 );
-            $recursive ^= 1;
         }
     );
 
@@ -1845,7 +1844,6 @@ sub preferences {
         toggled => sub {
             ClamTk::Prefs->set_preference( 'Thorough',
                 $deep_box->get_active ? 1 : 0 );
-            $thorough ^= 1;
         }
     );
 
@@ -1857,19 +1855,33 @@ sub preferences {
         toggled => sub {
             ClamTk::Prefs->set_preference( 'SizeLimit',
                 $size_box->get_active ? 1 : 0 );
-            $size_set ^= 1;
+        }
+    );
+
+    my $mounted_box = Gtk2::CheckButton->new_with_label(
+
+        # We're sprintf-ing this so that %s can
+        # expand to KIO or whatever KDE uses and it
+        # won't require a translation change
+        sprintf gettext('Scan samba-related directories %s'), "(gvfs, smb4k)"
+    );
+    $mounted_box->set_active(TRUE) if ( $prefs{Mounted} );
+    $scan_table->attach_defaults( $mounted_box, 0, 1, 4, 5 );
+    $mounted_box->signal_connect(
+        toggled => sub {
+            ClamTk::Prefs->set_preference( 'Mounted',
+                $mounted_box->get_active ? 1 : 0 );
         }
     );
 
     my $log_box = Gtk2::CheckButton->new_with_label(
         gettext('Keep a record of every scan') );
     $log_box->set_active(TRUE) if ( $prefs{SaveToLog} );
-    $scan_table->attach_defaults( $log_box, 0, 1, 4, 5 );
+    $scan_table->attach_defaults( $log_box, 0, 1, 5, 6 );
     $log_box->signal_connect(
         toggled => sub {
             ClamTk::Prefs->set_preference( 'SaveToLog',
                 $log_box->get_active ? 1 : 0 );
-            $save_log ^= 1;
         }
     );
 
@@ -1943,14 +1955,17 @@ sub preferences {
             if ( "ok" eq $dialog->run ) {
                 $dir = $dialog->get_filename;
                 if ( $dir eq "/" ) {
+
                     # Just in case someone clicks the root (/).
                     $dialog->destroy;
                     return;
                 }
+
                 # See if it's already included...
                 if ( !grep {/^$dir$/} split /;/,
                     $user_whitelist . $system_whitelist )
                 {
+
                     # If not, add to GUI...
                     push @{ $ex_slist->{data} }, $dir;
 
@@ -2245,6 +2260,7 @@ sub preferences {
 }
 
 sub proxy_non_block_status {
+
     # This is a non-blocking way to show success or failure
     # in the proxy configuration dialog.
     # I think muppet came up with this.
@@ -2268,7 +2284,7 @@ sub proxy_non_block_status {
 
 sub about {
     my $about = Gtk2::AboutDialog->new;
-    $about->set_authors("Dave M, 2004-2010\ndave.nerd <at> gmail.com");
+    $about->set_authors("Dave M, 2004-2011\ndave.nerd <at> gmail.com");
     $about->set_version( ClamTk::App->get_TK_version() );
     my $contributors = 'Please see http://clamtk.sf.net/credits.html';
     $about->set_translator_credits($contributors);
@@ -2287,8 +2303,7 @@ sub about {
     );
     $about->set_license(
         gettext(
-                  "ClamTk, (c) 2004-2010. All rights reserved.\n\n"
-                . "This program is free software; you can redistribute it\n"
+                  "This program is free software; you can redistribute it\n"
                 . "and/or modify it under the terms of either:\n\n"
                 . "a) the GNU General Public License as published by\n"
                 . "the Free Software Foundation; either version 1, or\n"
