@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2010 Dave M
+# ClamTk, copyright (C) 2004-2011 Dave M
 #
 # This file is part of ClamTk.
 #
@@ -13,14 +13,16 @@ package ClamTk::Prefs;
 
 use strict;
 #use warnings;    # disabled upon release
-$|++;
+$| = 1;
+
+use encoding 'utf8';
 
 use ClamTk::App;
 use ClamTk::Submit;
 
-use encoding 'utf8';
-
+use Digest::MD5 qw/md5_hex/;
 use File::Path 'mkpath';
+
 use Locale::gettext;
 use POSIX 'locale_h';
 textdomain('clamtk');
@@ -161,7 +163,7 @@ sub custom_prefs {
 
     for my $o (
         qw/SaveToLog ScanHidden SizeLimit
-        Thorough Recursive/
+        Thorough Recursive Mounted/
         )
     {
         if ( !exists $pkg{$o} ) {
@@ -323,7 +325,7 @@ sub restore {
                 return ( $p{$e}->{path}, $p{$e}->{perm} );
             }
         }
-        return -1;
+        return 0;
     }
 
     if ( $job eq 'add' ) {
@@ -362,7 +364,7 @@ sub restore {
 }
 
 sub restore_file_fix {
-    # This subroutine might only be in 4.25; for some reason the line
+    # This subroutine was only supposed to be in 4.25 because the line
     # 'next if ($e eq $wanted)' got dropped from 'sub restore' above.
     # So, we need to remove all the lines from the restore file that
     # don't have coresponding viruses in the quarantine directory.
@@ -371,10 +373,8 @@ sub restore_file_fix {
     my $quarantine = ClamTk::App->get_path('viruses');
     return unless ( -e $restore && -e $quarantine );
 
-    use Digest::MD5 qw/md5_hex/;
-
     # First, grab everything quarantined.
-    my @viruses = glob "$quarantine/*";
+    my @viruses = grep { -f $_ } glob "$quarantine/*";
 
     # Now, grab their md5sums
     my %p;
