@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2010 Dave M
+# ClamTk, copyright (C) 2004-2011 Dave M
 #
 # This file is part of ClamTk.
 #
@@ -13,7 +13,7 @@ package ClamTk::Results;
 
 use strict;
 #use warnings;    # disabled upon release
-$|++;
+$| = 1;
 
 use encoding 'utf8';
 
@@ -21,7 +21,6 @@ use File::Basename;
 use File::Copy;
 use Digest::MD5 'md5_hex';
 
-use Gtk2;
 use Gtk2::SimpleList;
 use Glib 'TRUE', 'FALSE';
 
@@ -76,8 +75,12 @@ sub display {
     $slist->set_reorderable(TRUE);
     map { $_->set_resizable(TRUE) } $slist->get_columns;
 
-    #map { $_->set_fixed_width(250) } $slist->get_columns;
-    #map { $_->set_sizing('fixed') } $slist->get_columns;
+    # Sorting the columns.
+    # Can't do it yet because it dorks up the sexy highlighting
+    # after quarantining or deleting a file.
+    #for ( 0 .. 2 ) {
+    #    $slist->get_column($_)->set_sort_column_id($_);
+    #}
 
     $slist->set(
         hover_selection => TRUE,
@@ -113,7 +116,7 @@ sub display {
             my $maildirs = join( '|',
                 '.thunderbird', '.mozilla-thunderbird',
                 '.evolution',   'Mail',
-                'kmail' );
+                'kmail',        "\.pst" );
 
             my $menu = Gtk2::Menu->new();
 
