@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2010 Dave M
+# ClamTk, copyright (C) 2004-2011 Dave M
 #
 # This file is part of ClamTk.
 #
@@ -16,7 +16,7 @@ package ClamTk::Schedule;
 
 use strict;
 #use warnings;    # disabled upon release
-$|++;
+$| = 1;
 
 use encoding 'utf8';
 
@@ -26,7 +26,6 @@ textdomain("clamtk");
 setlocale( LC_MESSAGES, "" );
 bind_textdomain_codeset( "clamtk", "UTF-8" );
 
-use Gtk2;
 use Glib 'TRUE', 'FALSE';
 
 # This should be under /usr/bin, but we'll check anyway.
@@ -50,6 +49,7 @@ sub schedule_dialog {
     $window = Gtk2::Window->new;
     $window->signal_connect( destroy => sub { Gtk2->main_quit; } );
     $window->set_title( gettext('Schedule') );
+    $window->set_position('center');
 
     $tt = Gtk2::Tooltips->new();
 
