@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2010 Dave M
+# ClamTk, copyright (C) 2004-2011 Dave M
 #
 # This file is part of ClamTk.
 #
@@ -14,15 +14,14 @@ package ClamTk::Submit;
 # better.  It's like one big hackjob.
 
 use strict;
-#use warnings;	#disabled upon release
-$|++;
+#use warnings;    #disabled upon release
+$| = 1;
 
 use LWP::UserAgent;
 use Encode 'from_to', 'decode';
 use Date::Calc 'Delta_Days';
 use File::Basename 'basename';
 
-use Gtk2 '-init';
 use Glib 'TRUE', 'FALSE';
 
 use Locale::gettext;
@@ -49,8 +48,7 @@ $file_path .= '/times';
 sub analysis {
     $sub_window = Gtk2::Window->new();
     $sub_window->signal_connect( destroy => sub { $sub_window->destroy } );
-    #$sub_window->set_size_request( 550, 420 );
-    #$sub_window->set_resizable(FALSE);
+    $sub_window->set_position('center');
     $sub_window->set_title( change( gettext('Submit a file for analysis') ) );
     if ( -e '/usr/share/pixmaps/clamtk.png' ) {
         $sub_window->set_default_icon_from_file(
@@ -352,7 +350,7 @@ sub finish {
 
     $bottombox->hide();
 
-    my $message;
+    my $message = "\n";
     if ( $status eq 'success' ) {
         $message = change( gettext('The submission was successful!') );
     } elsif ( $status eq 'known' ) {
@@ -370,9 +368,6 @@ sub finish {
                 'Unable to complete the submission. Please try again later.')
         );
     }
-    $message .= "\n\n";
-    $message
-        .= change( gettext('Thank you for helping the ClamAV project.') );
     $statusbar->set_text($message);
     clear();
     $close_btn->show();
@@ -382,6 +377,7 @@ sub create_view {
     my $view = Gtk2::TextView->new;
     $view->set_wrap_mode('word');
     $view->set_editable(FALSE);
+    $view->set_cursor_visible(FALSE);
     $view->set_indent(5);
 
     my $sw = Gtk2::ScrolledWindow->new;
