@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2010 Dave M
+# ClamTk, copyright (C) 2004-2011 Dave M
 #
 # This file is part of ClamTk.
 #
@@ -13,21 +13,19 @@ package ClamTk::Update;
 
 use strict;
 #use warnings;    # disabled upon release
-$|++;
+$| = 1;
 
 use LWP::UserAgent;
 
 use encoding 'utf8';
 
-use ClamTk::GUI;
-
 use Locale::gettext;
 use POSIX 'locale_h';
 textdomain('clamtk');
 setlocale( LC_MESSAGES, '' );
 bind_textdomain_codeset( 'clamtk', 'UTF-8' );
 
-use Gtk2;
+use Gtk2::SimpleList;
 use Glib 'TRUE', 'FALSE';
 
 my ( $win, $update_list );
@@ -150,7 +148,7 @@ sub update_dialog {
     );
     $cancel_btn->set_sensitive(FALSE);
 
-    # Warn the user not to do things as root
+    # Warn the user not to do things as root.
     if ( $> == 0 ) {
         my $warning_btn = Gtk2::Button->new_from_stock('gtk-dialog-warning');
         $hbox->add($warning_btn);
@@ -174,7 +172,7 @@ sub update_dialog {
     # We'll just keep about 30 lines for the regular user
     # since root will (hopefully) use the system's area which
     # will already do log rotation.
-    # The 30 lines will be useful for diagnosing problems
+    # The 30 lines will be useful for diagnosing problems.
     # Don't rotate if TruncateLog=0
     # This should only be expensive the first time or if the user
     # has automatic (cron) updates scheduled and doesn't come here often.
@@ -233,9 +231,9 @@ sub decision {
                 $update_list->{data}[$row][3] = gettext('Checking...');
                 $value = update_gui( 'dummy', 'not-startup' );
                 $update_list->{data}[$row][3] =
-                      $value == 1 ? gettext('GUI is current')
+                      $value == 1 ? gettext('Current')
                     : $value == 2 ? gettext('A newer version is available')
-                    : $value == 3 ? gettext('GUI is current')
+                    : $value == 3 ? gettext('Current')
                     : $value == 4 ? gettext('Check failed')
                     : $value == 5 ? gettext('Check failed')
                     :               '';
@@ -429,29 +427,22 @@ sub update_gui {
 }
 
 sub av_db_select {
-    my $choice = Gtk2::Dialog->new();
-    $choice->signal_connect( destroy => sub { $choice->destroy; } );
-    $choice->set_title( gettext('Antivirus Signatures') );
-    $choice->set_default_size( 375, 240 );
+    # This is an easy way for the user to switch back
+    # and forth their preferred way to update signatures.
 
-    if ( -e '/usr/share/pixmaps/clamtk.png' ) {
-        $choice->set_default_icon_from_file('/usr/share/pixmaps/clamtk.png');
-    } elsif ( -e '/usr/share/pixmaps/clamtk.xpm' ) {
-        $choice->set_default_icon_from_file('/usr/share/pixmaps/clamtk.xpm');
-    }
+    my $assistant = Gtk2::Assistant->new;
+    $assistant->set_default_size( -1, 240 );
 
-    my $view = Gtk2::TextView->new;
-    $view->set_wrap_mode('word');
-    $view->set_editable(FALSE);
-    $view->set_cursor_visible(FALSE);
-    $view->set_indent(5);
+    # What the user already has - and then chooses.
+    # We'll declare it now because of the button toggle action.
+    my $pref = '';
 
-    my $sw = Gtk2::ScrolledWindow->new;
-    $sw->set_shadow_type('etched-in');
-    $sw->set_policy( 'automatic', 'automatic' );
-    $sw->set_border_width(5);
+    # This is the first page.
+    my $abox = Gtk2::VBox->new( FALSE, 12 );
+    $abox->set_border_width(12);
 
-    # This is horrible and will be changed in 4.5 or 5.0.
+    # dlabel holds the main text (dialog)
+    my $dlabel = Gtk2::Label->new;
     my $dialog;
     $dialog .= gettext(
         'Please choose how you will update your antivirus signatures.');
@@ -465,84 +456,100 @@ sub av_db_select {
         'If your computer automatically receives updates, choose Automatic.'
         );
     $dialog .= "\n";
+    $dlabel->set_text($dialog);
 
-    my $buffer = $view->get_buffer;
-    $buffer->create_tag( 'mono', family => 'Monospace' );
-    $buffer->insert_with_tags_by_name( $buffer->get_start_iter, $dialog,
-        'mono' );
+    $abox->pack_start( $dlabel, FALSE, FALSE, 0 );
 
-    $sw->add($view);
-    $choice->vbox->pack_start( $sw, TRUE, TRUE, 0 );
+    # This ButtonBox will hold the manual/automatic option.
+    my $choice_bb = Gtk2::HButtonBox->new;
+    $abox->pack_start( $choice_bb, FALSE, FALSE, 0 );
+    # The 'center' layout might be nicer, but CentOS 5.5 has
+    # an older version of Gtk2 apparently, so we'll use 'spread'.
+    $choice_bb->set_layout('spread');
 
-    my $selection;
-
-    my $c_box = Gtk2::HButtonBox->new();
-    $choice->vbox->pack_start( $c_box, FALSE, FALSE, 0 );
-
-    # The {single,save}_btn buttons are
-    # set sensitive->FALSE when selected.
-    my ( $cancel_btn, $single_btn, $sys_btn, $save_btn, $quit_btn, $label );
-
-    # Cancel button
-    $cancel_btn = Gtk2::Button->new_from_stock('gtk-cancel');
-    $c_box->add($cancel_btn);
-    $cancel_btn->signal_connect( clicked => sub { $choice->destroy } );
-
-    # 'Single' button - user will update
-    $single_btn = Gtk2::Button->new( gettext('Manual') );
-    $c_box->add($single_btn);
-    $single_btn->signal_connect(
-        clicked => sub {
-            $single_btn->set_relief('none');
-            $sys_btn->set_relief('normal');
-            $selection = 'single';
-            $save_btn->set_sensitive(TRUE);
+    # Manually update the sigs
+    my $manual_btn =
+        Gtk2::RadioButton->new_with_label( undef, gettext('Manual') );
+    $choice_bb->add($manual_btn);
+    $manual_btn->signal_connect(
+        toggled => sub {
+            $pref = 'single';
         }
     );
 
-    # 'Shared' button - system updates
-    $sys_btn = Gtk2::Button->new( gettext('Automatic') );
-    $c_box->add($sys_btn);
-    $sys_btn->signal_connect(
-        clicked => sub {
-            $sys_btn->set_relief('none');
-            $single_btn->set_relief('normal');
-            $selection = 'shared';
-            $save_btn->set_sensitive(TRUE);
+    # Automatically update the sigs.
+    # This is the default, out-of-the-box choice.
+    my $auto_btn =
+        Gtk2::RadioButton->new_with_label( $manual_btn->get_group,
+        gettext('Automatic') );
+    $choice_bb->add($auto_btn);
+    $auto_btn->signal_connect(
+        toggled => sub {
+            $pref = 'shared';
         }
     );
 
-    # 'Save' button - not sensitive until
-    # 'single' or 'shared' is selected
-    $save_btn = Gtk2::Button->new_from_stock('gtk-save');
-    $c_box->add($save_btn);
-    $save_btn->signal_connect(
-        clicked => sub {
-            save($selection);
-            $label->set_text( gettext('Your preferences were saved.') );
-            $save_btn->set_sensitive(FALSE);
-            $single_btn->set_sensitive(FALSE);
-            $sys_btn->set_sensitive(FALSE);
-            $quit_btn->set_sensitive(TRUE);
+    # We'll start with what the user currently has selected
+    # as their update preference.
+    $pref = ClamTk::Prefs->get_preference('Update');
+    # Just in case, set it to 'shared' if nothing else:
+    $pref ||= 'shared';
+
+    if ( $pref eq 'shared' ) {
+        $auto_btn->set_active(TRUE);
+    } elsif ( $pref eq 'single' ) {
+        $manual_btn->set_active(TRUE);
+    }
+
+    $assistant->append_page($abox);
+    $assistant->set_page_title( $abox,
+        gettext('Antivirus signature options') );
+
+    # Although easy enough to use the ClamTk png for this,
+    # we'll just use default question/info images.
+    my $pixbuf = $abox->render_icon( 'gtk-dialog-question', 'dialog' );
+    $assistant->set_page_side_image( $abox, $pixbuf );
+
+    # The 'confirm' style is necessary for the 'apply' button.
+    $assistant->set_page_type( $abox, 'confirm' );
+    # There's an option already there, so set it complete no matter what.
+    $assistant->set_page_complete( $abox, TRUE );
+
+    # The Assistant's callbacks
+    $assistant->signal_connect(
+        cancel => sub {
+            $assistant->destroy;
+        }
+    );
+    $assistant->signal_connect(
+        close => sub {
+            $assistant->destroy;
+        }
+    );
+    $assistant->signal_connect(
+        apply => sub {
+            save($pref);
         }
     );
-    $save_btn->set_sensitive(FALSE);
 
-    # 'Quit' button
-    $quit_btn = Gtk2::Button->new_from_stock('gtk-quit');
-    $c_box->add($quit_btn);
-    $quit_btn->signal_connect( clicked => sub { $choice->destroy } );
-    $quit_btn->set_sensitive(FALSE);
+    # This is the second page: 'confirm' or confirmation.
+    my $comp_page = Gtk2::Alignment->new( 0.5, 0.5, 0.5, 0.0 );
 
-    $label = Gtk2::Label->new();
-    $choice->vbox->pack_start( $label, FALSE, FALSE, 0 );
+    # Not much else to say...
+    my $label = Gtk2::Label->new( gettext('Your preferences were saved.') );
+    $comp_page->add($label);
 
-    $view->show();
-    $choice->show_all();
-    $choice->run;
-    $choice->destroy;
+    $assistant->append_page($comp_page);
+    $assistant->set_page_title( $comp_page,
+        gettext('Antivirus signature options') );
+    $assistant->set_page_type( $comp_page, 'summary' );
+    $assistant->set_page_complete( $abox, TRUE );
 
-    ClamTk::GUI->set_sig_status();
+    # Now switch the image to 'info'.
+    $pixbuf = $abox->render_icon( 'gtk-dialog-info', 'dialog' );
+    $assistant->set_page_side_image( $comp_page, $pixbuf );
+
+    $assistant->show_all();
     return;
 }
 
